@@ -19,6 +19,7 @@ from fontmake import __version__
 from fontmake.font_project import FontProject
 from fontmake.errors import FontmakeError
 from ufo2ft.featureWriters import loadFeatureWriterFromString
+from ufo2ft import CFFOptimization
 
 
 def _loadFeatureWriters(parser, specs):
@@ -133,11 +134,20 @@ def main(args=None):
         help='Directory where to write instance UFOs. Default: '
              '"./instance_ufo". If value is "{tmp}", a temporary directory '
              'is created and removed at the end (for Glyphs sources only).')
+    outputGroup.add_argument(
+        '--validate-ufo', action='store_true',
+        help='Enable ufoLib validation on reading/writing UFO files. It is '
+             'disabled by default')
 
     contourGroup = parser.add_argument_group(title='Handling of contours')
     contourGroup.add_argument(
         '--keep-overlaps', dest='remove_overlaps', action='store_false',
         help='Do not remove any overlap.')
+    contourGroup.add_argument(
+        '--overlaps-backend', dest='overlaps_backend', metavar="BACKEND",
+        choices=("booleanOperations", "pathops"), default="booleanOperations",
+        help='Select library to remove overlaps. Choose between: %(choices)s '
+             '(default: %(default)s)')
     contourGroup.add_argument(
         '--keep-direction', dest='reverse_direction', action='store_false',
         help='Do not reverse contour direction when output is ttf or '
@@ -156,6 +166,13 @@ def main(args=None):
              'is less than or equal to the tolerance. By default, all floats '
              'are rounded to integer (tolerance 0.5); 0 disables rounding.'
     )
+    contourGroup.add_argument(
+        '--optimize-cff',
+        type=lambda s: CFFOptimization(int(s)),
+        default=CFFOptimization.SUBROUTINIZE,
+        help='0 disables all optimizations; 1 specializes the CFF charstring '
+             'operators; 2 (default) also enables subroutinization'
+    )
 
     layoutGroup = parser.add_argument_group(title='Handling of OpenType Layout')
     layoutGroup.add_argument(
@@ -201,12 +218,13 @@ def main(args=None):
     subroutinizeGroup = parser.add_mutually_exclusive_group(required=False)
     subroutinizeGroup.add_argument(
         '-s', '--subroutinize', action='store_true',
-        help='Optimize CFF table using compreffor (default)')
+        help='Optimize CFF table using compreffor (default) [DEPRECATED: use '
+             '--optimize-cff option instead]')
     subroutinizeGroup.add_argument(
         '-S', '--no-subroutinize', dest='subroutinize', action='store_false')
 
     parser.set_defaults(use_production_names=None, subset=None,
-                        subroutinize=True)
+                        subroutinize=None)
 
     logGroup = parser.add_argument_group(title='Logging arguments')
     logGroup.add_argument(
@@ -241,7 +259,8 @@ def main(args=None):
 
     try:
         project = FontProject(timing=args.pop('timing'),
-                              verbose=args.pop('verbose'))
+                              verbose=args.pop('verbose'),
+                              validate_ufo=args.pop('validate_ufo'))
 
         if glyphs_path:
             with _make_tempdirs(parser, args):
