@@ -39,6 +39,7 @@ except ImportError:
         return re.match("(?:" + regex + r")\Z", string, flags=flags)
 
 from defcon import Font
+from defcon.objects.base import setUfoLibReadValidate, setUfoLibWriteValidate
 from fontTools.misc.py23 import tobytes, basestring, zip
 from fontTools.misc.loggingTools import configLogger, Timer
 from fontTools.misc.transform import Transform
@@ -50,7 +51,9 @@ from fontTools import designspaceLib
 from fontTools.varLib.interpolate_layout import interpolate_layout
 from ufo2ft import compileOTF, compileTTF, compileInterpolatableTTFs
 from ufo2ft.featureCompiler import FeatureCompiler
+from ufo2ft.featureWriters import loadFeatureWriters, FEATURE_WRITERS_KEY
 from ufo2ft.util import makeOfficialGlyphOrder
+from ufo2ft import CFFOptimization
 
 from fontmake.errors import FontmakeError, TTFAError
 from fontmake.ttfautohint import ttfautohint
@@ -89,12 +92,17 @@ def _deprecated(func):
 class FontProject(object):
     """Provides methods for building fonts."""
 
-    def __init__(self, timing=False, verbose='INFO'):
+    def __init__(self, timing=False, verbose='INFO', validate_ufo=False):
         logging.basicConfig(level=getattr(logging, verbose.upper()))
         logging.getLogger('fontTools.subset').setLevel(logging.WARNING)
         if timing:
             configLogger(logger=timer.logger, level=logging.DEBUG)
 
+        logger.debug("ufoLib UFO validation is %s",
+                     "enabled" if validate_ufo else "disabled")
+        setUfoLibReadValidate(validate_ufo)
+        setUfoLibWriteValidate(validate_ufo)
+
     @timer()
     def build_master_ufos(self, glyphs_path, designspace_path=None,
                           master_dir=None, instance_dir=None,
@@ -104,8 +112,12 @@ class FontProject(object):
 
         if master_dir is None:
             master_dir = self._output_dir('ufo')
+        if not os.path.isdir(master_dir):
+            os.mkdir(master_dir)
         if instance_dir is None:
             instance_dir = self._output_dir('ufo', is_instance=True)
+        if not os.path.isdir(instance_dir):
+            os.mkdir(instance_dir)
 
         font = glyphsLib.GSFont(glyphs_path)
 
@@ -126,8 +138,6 @@ class FontProject(object):
             # no need to also set the relative 'filename' attribute as that
             # will be auto-updated on writing the designspace document
             source.path = ufo_path
-            if not os.path.isdir(master_dir):
-                os.makedirs(master_dir)
             source.font.save(ufo_path)
 
         if designspace_path is None:
@@ -289,9 +299,11 @@ class FontProject(object):
                   autohint=None,
                   subset=None,
                   use_production_names=None,
-                  subroutinize=False,
+                  subroutinize=None,  # deprecated
+                  optimize_cff=CFFOptimization.NONE,
                   cff_round_tolerance=None,
                   remove_overlaps=True,
+                  overlaps_backend=None,
                   reverse_direction=True,
                   conversion_error=None,
                   feature_writers=None,
@@ -323,6 +335,8 @@ class FontProject(object):
                 between only round floats which are close to their integral
                 part within the tolerated range. Ignored if ttf=True.
             remove_overlaps: If True, remove overlaps in glyph shapes.
+            overlaps_backend: name of the library to remove overlaps. Can be
+                either "booleanOperations" (default) or "pathops".
             reverse_direction: If True, reverse contour directions when
                 compiling TrueType outlines.
             conversion_error: Error to allow when converting cubic CFF contours
@@ -347,6 +361,21 @@ class FontProject(object):
         if output_path is not None and len(ufos) > 1:
             raise ValueError("output_path requires a single input")
 
+        if subroutinize is not None:
+            import warnings
+
+            warnings.warn(
+                "the 'subroutinize' argument is deprecated, use 'optimize_cff'",
+                UserWarning
+            )
+            if subroutinize:
+                optimize_cff = CFFOptimization.SUBROUTINIZE
+            else:
+                # for b/w compatibility, we still run the charstring specializer
+                # even when --no-subroutinize is used. Use the new --optimize-cff
+                # option to disable both specilization and subroutinization
+                optimize_cff = CFFOptimization.SPECIALIZE
+
         ext = 'ttf' if ttf else 'otf'
 
         if interpolate_layout_from is not None:
@@ -386,7 +415,8 @@ class FontProject(object):
                 ufos,
                 ttf,
                 removeOverlaps=remove_overlaps,
-                optimizeCFF=subroutinize,
+                overlapsBackend=overlaps_backend,
+                optimizeCFF=optimize_cff,
                 roundTolerance=cff_round_tolerance,
                 **compiler_options)
 
@@ -551,6 +581,7 @@ class FontProject(object):
             self, designspace_path, interpolate=False,
             masters_as_instances=False,
             interpolate_binary_layout=False, round_instances=False,
+            feature_writers=None,
             **kwargs):
         """Run toolchain from a MutatorMath design space document.
 
@@ -585,6 +616,14 @@ class FontProject(object):
         from glyphsLib.interpolation import apply_instance_data
         from mutatorMath.ufo.document import DesignSpaceDocumentReader
 
+        designspace = designspaceLib.DesignSpaceDocument.fromfile(designspace_path)
+
+        # if no --feature-writers option was passed, check in the designspace's
+        # <lib> element if user supplied a custom featureWriters configuration;
+        # if so, use that for all the UFOs built from this designspace
+        if feature_writers is None and FEATURE_WRITERS_KEY in designspace.lib:
+            feature_writers = loadFeatureWriters(designspace)
+
         ufos = []
         reader = DesignSpaceDocumentReader(designspace_path, ufoVersion=3,
                                            roundGeometry=round_instances,
@@ -594,7 +633,7 @@ class FontProject(object):
         if interpolate:
             logger.info('Interpolating master UFOs from designspace')
             if isinstance(interpolate, basestring):
-                instances = self._search_instances(designspace_path,
+                instances = self._search_instances(designspace,
                                                    pattern=interpolate)
                 for instance_name in instances:
                     reader.readInstance(("name", instance_name))
@@ -620,6 +659,7 @@ class FontProject(object):
             is_instance=(interpolate or masters_as_instances),
             interpolate_layout_from=interpolate_layout_from,
             interpolate_layout_dir=interpolate_layout_dir,
+            feature_writers=feature_writers,
             **kwargs)
 
     def run_from_ufos(
@@ -640,14 +680,19 @@ class FontProject(object):
 
         if set(output) == set(['ufo']):
             return
-        if hasattr(ufos[0], 'path'):
-            ufo_paths = [ufo.path for ufo in ufos]
+
+        # the `ufos` parameter can be a list of UFO objects
+        # or it can be a path (string) with a glob syntax
+        ufo_paths = []
+        if isinstance(ufos, basestring):
+            ufo_paths = glob.glob(ufos)
+            ufos = [Font(x) for x in ufo_paths]
+        elif isinstance(ufos, list):
+            # ufos can be either paths or open Font objects, so normalize them
+            ufos = [Font(x) if isinstance(x, basestring) else x for x in ufos]
+            ufo_paths = [x.path for x in ufos]
         else:
-            if isinstance(ufos, str):
-                ufo_paths = glob.glob(ufos)
-            else:
-                ufo_paths = ufos
-            ufos = [Font(path) for path in ufo_paths]
+            raise FontmakeError('UFOs parameter is neither a defcon.Font object, a path or a glob, nor a list of any of these.', ufos)
 
         need_reload = False
         if 'otf' in output:
@@ -690,9 +735,7 @@ class FontProject(object):
                     shutil.rmtree(tempdir)
 
     @staticmethod
-    def _search_instances(designspace_path, pattern):
-        designspace = designspaceLib.DesignSpaceDocument()
-        designspace.read(designspace_path)
+    def _search_instances(designspace, pattern):
         instances = OrderedDict()
         for instance in designspace.instances:
             # is 'name' optional? 'filename' certainly must not be
@@ -704,8 +747,17 @@ class FontProject(object):
 
     def _font_name(self, ufo):
         """Generate a postscript-style font name."""
-        return '%s-%s' % (ufo.info.familyName.replace(' ', ''),
-                          ufo.info.styleName.replace(' ', ''))
+        family_name = (
+            ufo.info.familyName.replace(' ', '')
+            if ufo.info.familyName is not None
+            else 'None'
+        )
+        style_name = (
+            ufo.info.styleName.replace(' ', '')
+            if ufo.info.styleName is not None
+            else 'None'
+        )
+        return '%s-%s' % (family_name, style_name)
 
     def _output_dir(self, ext, is_instance=False, interpolatable=False,
                     autohinted=False, is_variable=False):
