@@ -24,11 +24,21 @@ function check_failure() {
 
 for src in 'DesignspaceTest' 'AvarDesignspaceTest'; do
     cd "${src}"
+    mkdir -p instance_ufo
     fontmake -i -m "${src}.designspace"
     check_failure "${src} failed to build"
     cd ..
 done
 
+# also test the masters_as_instance parameter
+for src in 'DesignspaceTest' 'AvarDesignspaceTest'; do
+    cd "${src}"
+    mkdir -p instance_ufo
+    fontmake -i -M -m "${src}.designspace"
+    check_failure "${src} failed to build"
+    cd ..
+done
+
 for src in 'InterpolateLayoutTest'; do
     cd "${src}"
     fontmake -g "${src}.glyphs" --mti-source "${src}.plist" --no-production-names
@@ -46,5 +56,7 @@ fontmake -g TestSubset.glyphs -i "Test Subset Regular" -o ttf otf
 check_failure "TestSubset.glyphs failed to build"
 
 python test_output.py
+check_failure 'fontmake output incorrect'
+
 python test_arguments.py
 check_failure 'fontmake output incorrect'
