@@ -19,10 +19,13 @@ import unittest
 from fontTools.ttLib import TTFont
 
 
+HERE = os.path.dirname(__file__)
+
+
 class TestAvarOutput(unittest.TestCase):
     def _get_font(self):
         return TTFont(os.path.join(
-            'AvarDesignspaceTest', 'instance_ttf', 'MyFont-Regular.ttf'))
+            HERE, 'AvarDesignspaceTest', 'instance_ttf', 'MyFont-Regular.ttf'))
 
     def test_weight_classes(self):
         font = self._get_font()
@@ -31,10 +34,27 @@ class TestAvarOutput(unittest.TestCase):
     def test_interpolation(self):
         font = self._get_font()
         glyph_set = font.getGlyphSet()
-        glyph = glyph_set['uni006C']._glyph
+        glyph = glyph_set['l']._glyph
         self.assertEqual(glyph.xMin, 50)
         self.assertEqual(glyph.xMax, 170)
 
 
+class TestSubsetOutput(unittest.TestCase):
+
+    def test_glyph_order(self):
+        for ext in ("otf", "ttf"):
+            font = TTFont(
+                os.path.join(
+                    HERE,
+                    "instance_%s" % ext,
+                    "Test-SubsetRegular.%s" % ext
+                )
+            )
+            self.assertEqual(
+                font.getGlyphOrder(),
+                [".notdef", "space", "A", "C", "B"],
+            )
+
+
 if __name__ == '__main__':
     unittest.main()
